function [x,iter] = conjgrad ( A, b, x0, TOL, Nmax,P )

%CONJ_GRAD    approximate the solution of the linear system Ax = b by applying
%             the Preconditioned (by P matrix) conjugate gradient method
%
%     calling sequences:
%             x = conj_grad ( A, b, x, TOL, Nmax )
%             conj_grad ( A, b, x, TOL, Nmax )
%
%     inputs:
%             A       coefficient matrix for linear system - must be a
%                     square matrix
%             b       right-hand side vector for linear system - must be
%                     a column vector
%             x       column vector containing initial guess for solution of 
%                     linear system
%             TOL     convergence tolerance - applied to Euclidean norm of
%                     residual vector
%             NMax    maximum number of iterations to be performed
%             P       preconditioner (no preconditioner=> P is the identity matrix )
%
%     output:
%             x       approximate solution of linear system
%             iter    number of iterations
%

flag=0;
iter=0;
x=x0';
normb=norm(b);
if (normb==0), normb=1.0;end
r = b - A * x;
err=norm(r)/normb;
if (err < TOL) return, end
for n=1:Nmax
    z=P\r; rho=(r'*z);
    if n>1
        beta=rho/rho1; p=z+beta*p;
    else
        p=z;
    end
    q=A*p;
    alpha=rho/(p'*q);
    x=x+alpha*p;
    r=r-alpha*q;
    err=norm(r)/normb;
    if (err < TOL)  iter=n; break; end
    rho1=rho;
end
if (err > TOL) 
    flag=1, 
    disp ( 'conj_grad error: maximum number of iterations exceeded' );
end
x=x';

